From 6fe2ac02d68de95ab4fce7dcea676bcd24866f45 Mon Sep 17 00:00:00 2001
From: Petri Hintukainen <phintuka@gmail.com>
Date: Thu, 20 Aug 2015 14:52:07 +0300
Subject: [PATCH 6/7] bluray: synchronize BD-J overlay open/close

BD-J overlay is created from JVM thread
---
 modules/access/bluray.c | 13 +++++++++++++
 1 file changed, 13 insertions(+)

diff --git a/modules/access/bluray.c b/modules/access/bluray.c
index d4a0d0a..57fa6ae 100644
--- a/modules/access/bluray.c
+++ b/modules/access/bluray.c
@@ -143,6 +143,8 @@ struct  demux_sys_t
     bool                b_popup_available;
     mtime_t             i_still_end_time;
 
+    vlc_mutex_t         bdj_overlay_lock; /* used to lock BD-J overlay open/close while overlays are being sent to vout */
+
     /* */
     vout_thread_t       *p_vout;
 
@@ -354,6 +356,7 @@ static int blurayOpen(vlc_object_t *object)
     }
 
     vlc_mutex_init(&p_sys->pl_info_lock);
+    vlc_mutex_init(&p_sys->bdj_overlay_lock);
 
     /* Warning the user about AACS/BD+ */
     const BLURAY_DISC_INFO *disc_info = bd_get_disc_info(p_sys->bluray);
@@ -516,6 +519,7 @@ static void blurayClose(vlc_object_t *object)
     TAB_CLEAN(p_sys->i_title, p_sys->pp_title);
 
     vlc_mutex_destroy(&p_sys->pl_info_lock);
+    vlc_mutex_destroy(&p_sys->bdj_overlay_lock);
 
     free(p_sys->psz_bd_path);
     free(p_sys);
@@ -1072,14 +1076,19 @@ static void blurayDrawArgbOverlay(demux_t *p_demux, const BD_ARGB_OVERLAY* const
 static void blurayArgbOverlayProc(void *ptr, const BD_ARGB_OVERLAY *const overlay)
 {
     demux_t *p_demux = (demux_t*)ptr;
+    demux_sys_t *p_sys = p_demux->p_sys;
 
     switch (overlay->cmd) {
     case BD_ARGB_OVERLAY_INIT:
+        vlc_mutex_lock(&p_sys->bdj_overlay_lock);
         blurayInitArgbOverlay(p_demux, overlay->plane, overlay->w, overlay->h);
+        vlc_mutex_unlock(&p_sys->bdj_overlay_lock);
         break;
     case BD_ARGB_OVERLAY_CLOSE:
+        vlc_mutex_lock(&p_sys->bdj_overlay_lock);
         blurayClearOverlay(p_demux, overlay->plane);
         blurayCloseOverlay(p_demux, overlay->plane);
+        vlc_mutex_unlock(&p_sys->bdj_overlay_lock);
         break;
     case BD_ARGB_OVERLAY_FLUSH:
         blurayActivateOverlay(p_demux, overlay->plane);
@@ -1665,6 +1674,8 @@ static int blurayDemux(demux_t *p_demux)
         }
     }
 
+    vlc_mutex_lock(&p_sys->bdj_overlay_lock);
+
     for (int i = 0; i < MAX_OVERLAY; i++) {
         bluray_overlay_t *ov = p_sys->p_overlays[i];
         if (!ov) {
@@ -1684,6 +1695,8 @@ static int blurayDemux(demux_t *p_demux)
         }
     }
 
+    vlc_mutex_unlock(&p_sys->bdj_overlay_lock);
+
     if (nread <= 0) {
         block_Release(p_block);
         if (nread < 0)
-- 
2.1.4

